#ifndef PPSSUMM_FILENAME
#define PPSSUMM_FILENAME
#include <fstream>
#include <string>
#include <cstdlib>
#include "Filename_typedef.h"
#include "Instrument.h"
#include "Src_num.h"
#include "Band.h"
#include "ProductClass.h"
#include "Rgsorder.h"
#include "Osw_id.h"

bool fileExists( const char* c );

namespace ppssumm_internal
{

class Filename 
{
    public:
    Filename() : _filename("") { }
    Filename( const char* c ) : _filename( c ) {}
    Filename( const std::string& s ) : _filename( s ) {}
    Filename( const Filename& c ) : _filename( c._filename ) { }

//P0116320801PNX000BKGMAP5000.FTZ
// 123456789 123456789 123456789 12345
//123456789 123456789 123456789 12345

    /* Access components of the filename */
    Instrument instrument() const 
    {
	return (_filename.size() == 31)
	    ? _filename.substr( 11 , 2 )
	    : ""
	;
    }

    Exp_id exp_id() const
    {
	return  (_filename.size() == 31)
	    ? _filename.substr( 13 , 4 )
	    : ""
	;
    }

    ProductClass productclass() const
    {
	std::string pc = (_filename.size() == 31)
	    ? _filename.substr( 17 , 6 )
	    : ""
	;
	return pc;
    }

    // The next four refer to the same position but vary with instrument.
    Osw_id osw_id() const
    {
    return  (_filename.size() == 31)
        ? Osw_id(_filename.substr( 23 , 1 ))
        : Osw_id()
    ;
    }

    Band band() const
    {
    return  (_filename.size() == 31)
        ? Band( _filename.substr( 23 , 1 ) )
        : Band(0)
    ;
    }

    Rgsorder rgsorder() const
    {
	return  (_filename.size() == 31)
	    ? Rgsorder( _filename.substr( 23 , 1 ) )
	    : Rgsorder()
	;
    }

    Src_num src_num() const
    {
	return (_filename.size() == 31)
	    ? Src_num(_filename.substr( 24 , 3 ))
	    : Src_num()
	;
    }

    Format format() const
    {
	Format f("");
	std::string::size_type sep = _filename.find(".");
	if ( sep != std::string::npos ) 
	{
	    f = _filename.substr( sep+1 , _filename.size() - sep ) ;
	}
	return f;
    }

    /* Return the filename as type std::string */
    std::string asString() const { return _filename; }

    int order() const
    {
	// return a numerical value based on name to make sorting easier.
	int r = 0;

	if ( instrument() == "EMOS1" || instrument() == "EMOS2" || instrument() == "EPN" )	
	{
	    ProductClass pc = productclass();
	    if ( pc == "SRSPEC" ) // EPIC SOURCE SPECTRUM 
		r = -8;
	    else if ( pc == "SPCPLT" ) // EPIC SOURCE SPECTRUM PLOT
		r = -6;
	    else if ( pc == "BGSPEC" ) // EPIC SOURCE BACKGROUND SPECTRUM
		r = -7;
	    else if ( pc == "SRCREG" ) // EPIC SOURCE REGION
		r = -2;
	    else if ( pc == "SRCARF" ) // EPIC ANCILLARY RESPONSE FUNCTION
		r = -1;
	    else if ( pc == "SFFTPL" ) // EPIC SOURCE FFT PLOT
		r = -3 ;
	    else if ( pc == "STSPLT" ) // EPIC SOURCE TIMESERIES PLOT
		r = -4;
	    else if ( pc == "SRCTSR" ) // EPIC SOURCE TIMESERIES
		r = -5;
	    else if ( pc == "MIEVLI" ) // EPIC MOS EVENT LIST
		r = -2;
	    else if ( pc == "PIEVLI" ) // EPIC PN EVENT LIST
		r = -2;
	    else if ( pc == "FBKTSR" ) // Flare background time series
		r = -1;
	    else
		r = 0;
	}
	return r;
    }

    // Operators
    bool operator==(const Filename& s) const
    {
	return _filename == s._filename;
    }
    bool operator<(const Filename& s) const
    {

	int x = order();
	int y = s.order();
	bool r = ( x < y )
	    || ( ( x == y ) && ( _filename < s._filename ) )
	;
	return r;

    }

    friend std::ostream &operator<<(std::ostream& os , const Filename& s)
    {

        /* Decide if to turn output into an html link or not 
         * Make links to these types if the file exists: FTZ , FIT , PDF , PNG , GIF , HTM
         * Always make a link to files of class SUMMAR and XCORRE.  If these do not exist a dummy page will be created.
         */
        bool make_link = 0;
        Format extn = s.format();
        if ( extn == "FTZ" 
	    || extn == "FIT" 
	    || extn == "ASC" 
	    || extn == "ASZ" 
	    || extn == "PDF" 
	    || extn == "PNG"
	    || extn == "GIF"
	    || extn == "HTM"
	) make_link = 1;

	// Force a link to these file types.  If they do not exist now
	// they will be created later.
        if ( !make_link )
        {
	    ProductClass p = s.productclass();
	    if ( p == "SUMMAR" || p == "XCORRE" )
		make_link = 1
	    ;
        }

        /* Now generate output */
        if ( make_link )
        {
	    os << "<a href=\"" << s._filename << "\">" << s._filename << "</a>";
        }
        else
        {
	    os << s._filename;
        }
        return os;
    }

    private:
    std::string _filename;
};

class MakeFilename
{
    public:
    Filename operator()( const Filename& s ) { return Filename( s ); }
    Filename operator()( const char* s ) { return Filename(s); }
    Filename operator()( const std::string& s ) { return Filename(s); }
};
/* 
 * Override the default less function.
 * This allows us to sort Filenames eg. std::list<Filename> foo.sort();
 * differently from strings
 */
/*
namespace std
{
    template<>
    class less<Filename>
    {
    public:
    bool operator()(const Filename& first , const Filename& second)
    {
        return second > first; 
    }
    };
}
*/
}
#endif
